/* ------------------------------------------------------------------------ */
/* ETCHSKCH.c - Etch-a-Sketch Software Toy                                  */
/*                                                                          */
/* Wayne Parham                                                             */
/*                                                                          */
/* wayne@parhamdata.com                                                     */
/* ------------------------------------------------------------------------ */

#include "stdio.h"

#define       CR        0x0D
#define       LF        0x0A
#define       BS        0x08
#define       ESC       0x1B
#define       CSI       '['
#define       ctrlC     0x03    /* Ctrl-C */
#define       ctrlQ     0x11    /* Quit   */
#define       ctrlX     0x18    /* Exit   */

int inchar()
{
   return( bios( 3 ) );
}

outchar( c )
int c;
{
   bios( 4, c );
}

cls()
{
   printf("\033[2J");
}

curpos(r,c) int r,c;
{
   printf("\033[%d;%dH",r,c);
}

up()
{
   outchar( ESC );
   outchar( CSI );
   outchar( 'A' );
}

down()
{
   outchar( ESC );
   outchar( CSI );
   outchar( 'B' );
}

right()
{
   outchar( ESC );
   outchar( CSI );
   outchar( 'C' );
}

left()
{
   outchar( ESC );
   outchar( CSI );
   outchar( 'D' );
}

main()
{
   int c, e, d, w;

   cls();

   puts( "\nThis  is a software version of the popular Etch-a-Sketch toy.  Use  the  cursor" );
   puts( "control, numeric keypad or selected keys to choose the direction of movement.\n" );

   puts( "The keys used to control the program are as follows:\n" );

   puts( "        W           I           8" );
   puts( "      A   S       J   L       4   6" );
   puts( "        Z           M           2\n" );

   puts( "Any  of  those keys can be used to move the cursor, as can the  cursor  control" );
   puts( "keys.  Use whichever is most convenient.  The 'C' key will clear the screen.\n" );

   puts( "The program is terminated by pressing the 'B' key or <ctrl-c>.\n" );

   puts( "Press 'ENTER' to continue." );

   while( inchar() != CR )
   {}

   cls();   
   curpos(1,1);

   w = 1;

   while( 1 ) {

      c = inchar();
      if( c == ESC ) {
         e = inchar();
         if( e == CSI ) {
            d = inchar();
            switch( d ) {
               case 'A':
                  /* cursor UP */
                  break;
               case 'B':
                  /* cursor DOWN */
                  break;
               case 'C':
                  /* cursor RIGHT */
                  if( w < 80 )
                     w++;
                  break;
               case 'D': 
                  /* cursor LEFT */
                  if( w > 1 )
                     w--;
                  break;
            }
            printf( "%c%c%c.", c, e, d );
            if( (w > 0) && (w < 80) )
               outchar( BS );
         }
      } else {
         switch( c ) {
            case ctrlC:
            case ctrlQ:
            case ctrlX:
            case 'b':
            case 'B':
               cls();
               exit();
            case 'c':
            case 'C':
               w = 1;
               cls();
               curpos(1,1);
            case '8':
            case 'w':
            case 'W':
            case 'i':
            case 'I':
               up();
               outchar( '.' );
               if( (w > 0) && (w < 80) )
                  outchar( BS );
               break;
            case '2':
            case 'z':
            case 'Z':
            case 'm':
            case 'M':
               down();
               outchar( '.' );
               if( (w > 0) && (w < 80) )
                  outchar( BS );
               break;
            case '4':
            case 'a':
            case 'A':
            case 'j':
            case 'J':
               left();
               outchar( '.' );
               if( w > 1 )
                  w--;
               if( (w > 0) && (w < 80) )
                  outchar( BS );
               break;
            case '6':
            case 's':
            case 'S':
            case 'k':
            case 'K':
               right();
               outchar( '.' );
               if( w < 80 )
                  w++;
               if( (w > 0) && (w < 80) )
                  outchar( BS );
               break;
         }
      }
   }
}
