/* Mand.c                                        */
/*                                               */
/* Generates ASCII output of the Mandelbrot set  */
/* using  (long)  integer math from the  BDS  C  */
/* library  that  represents longs  as  char[4]  */
/*                                               */
/* Written by Wayne Parham                       */


#include <stdio.h>


#define DISPLAY_COLS    80
#define DISPLAY_ROWS    23
#define INIT_LEFT     -560  /* (x size 800, 80 steps) */
#define INIT_RIGHT     240
#define INIT_TOP       220  /* (y size 460, 23 steps) */
#define INIT_BOTTOM   -240
#define INIT_X_INCR     10  /* (step ratio 2:1, y:x ) */
#define INIT_Y_INCR     20
#define INIT_MAX_ITER   50


void intro() {
   puts( "\n" );
   puts( "The Mandelbrot set includes all two-dimensional c values for which the function\n" );
   puts( "z = z^2 + c does not diverge when iterated from z = 0.  It was named in tribute\n" );
   puts( "to the mathematician Benoit Mandelbrot, a pioneer of fractal geometry.   He was\n" );
   puts( "first  to see a visualization of the set on March 1, 1980, at IBM's  Thomas  J.\n" );
   puts( "Watson Research Center in Yorktown Heights, New York.\n\n" );
   puts( "The  Mandelbrot  set  has  its  origin  in  complex  dynamics,  a  field  first\n" );
   puts( "investigated by the French mathematicians Pierre Fatou and Gaston Julia at  the\n" );
   puts( "beginning of the 20th century. This fractal was first defined and drawn in 1978\n" );
   puts( "by Robert W. Brooks and Peter Matelski as part of a study of Kleinian groups.\n\n" );
   puts( "Mandelbrot  studied the parameter space of quadratic polynomials in an  article\n" );
   puts( "that  appeared  in 1980.  The mathematical study of the Mandelbrot  set  really\n" );
   puts( "began with work by the mathematicians Adrien Douady and John H. Hubbard (1985),\n" );
   puts( "who  established many of its fundamental properties and named the set in  honor\n" );
   puts( "of Mandelbrot for his influential work in fractal geometry.\n\n" );
   puts( "This  program  displays the set on an ASCII terminal, much  like  the  earliest\n" );
   puts( "computers did.  It is a throwback to the computing technologies of  yesteryear.\n\n" );
}


void mandelbrot() {
   char c[4];
   char n[4];
   char cx[4];
   char cy[4];
   char zx[4];
   char zy[4];
   char zxx[4];
   char zyy[4];
   char zxzy[4];
   char zxxpluszyy[4];
   char zxxminuszyy[4];

   char boundL[4];
   char boundR[4];
   char boundT[4];
   char boundB[4];
   char x_incr[4];
   char y_incr[4];
   char max[4];

   char s[40];

   char p;
   int  i;

   itol( boundL, INIT_LEFT );
   itol( boundR, INIT_RIGHT );
   itol( boundT, INIT_TOP );
   itol( boundB, INIT_BOTTOM );
   itol( x_incr, INIT_X_INCR );
   itol( y_incr, INIT_Y_INCR );
   itol( max,    INIT_MAX_ITER );

   i = 0;                       /* for( cy=boundT; cy > boundB; cy -= y_incr ) {    */
                                /*    for( cx=boundL; cx < boundR; cx += x_incr ) { */

   for( lassign( cy, boundT );  lcomp( cy, boundB ) == 1;  lsub( cy, cy, y_incr ) ) {
      putchar( '\n' );
      for( lassign( cx, boundL );  lcomp( cx, boundR) == -1;  ladd( cx, cx, x_incr ) ) {

         itol(  n, 0 );                               /* n  = 0;                    */
         itol( zy, 0 );                               /* zy = 0;                    */
         itol( zx, 0 );                               /* zx = 0;                    */

         p  = ' ';

	 while( lcomp( n, max ) == -1 ) {             /* while( n < max ) {         */ 

            itol( c, 200 );

            lmul( zxx, zx, zx );
            ldiv( zxx, zxx, c );                      /* zxx = zx * zx / 200;       */

            lmul( zyy, zy, zy );
            ldiv( zyy, zyy, c );                      /* zyy = zy * zy / 200;       */

            itol( c, 800 );
            ladd( zxxpluszyy, zxx, zyy );

            if( lcomp( zxxpluszyy, c ) == 1 ) {       /* if( (zxx + zyy) > 800 ) {  */

               itol( c, 9 );
               if( lcomp( n, c ) == 1 ) {             /* if( n > 9 ) {              */
                  p = '0';                            /*    (char) p = '0'          */   
               } else {                               /* } else {                   */
                  p = ltou(n) + 48;                   /*    (char) p = n + 48       */
               }                                      /* }    (an ASCII conversion) */

               lassign(  n, max );                    /* n = max;                   */
            }

            itol( c, 100 );
            lmul( zxzy, zx, zy );
            ldiv( zxzy, zxzy, c);                     /* zy = zx * zy / 100 + cy;   */
            ladd( zy, zxzy, cy );

            lsub( zxxminuszyy, zxx, zyy );
            ladd( zx, zxxminuszyy, cx );              /* zx = zxx - zyy + cx;       */

            itol( c, 1 );
            ladd( n, n, c );                          /* n++;                       */
         }
         putchar( p );
      }

   }

}


void pressKey() {
   puts( "Press any key to continue.\n" );
   getchar();
}


void main() {
   intro();
   pressKey();
   mandelbrot();
}
